@@ -11,7 +11,7 @@ from itertools import repeat
 from rq.exceptions import NoSuchJobError
 from rq.job import Job
 from rq.queue import Queue
-from rq.utils import backend_class
+from rq.utils import backend_class, import_attribute
 
 from redis import WatchError
 
@@ -41,8 +41,10 @@ class Scheduler(object):
         self.log = logger
         self._lock_acquired = False
         self.job_class = backend_class(self, 'job_class', override=job_class)
-        self.queue_class = backend_class(self, 'queue_class',
-                                         override=queue_class)
+        self.queue_class_name = None
+        if isinstance(queue_class, str):
+            self.queue_class_name = queue_class
+        self.queue_class = backend_class(self, 'queue_class', override=queue_class)
         self.name = name or uuid4().hex
 
     @property
@@ -127,7 +129,7 @@ class Scheduler(object):
 
     def _create_job(self, func, args=None, kwargs=None, commit=True,
                     result_ttl=None, ttl=None, id=None, description=None,
-                    queue_name=None, timeout=None, meta=None, depends_on=None):
+                    queue_name=None, timeout=None, meta=None, depends_on=None, on_success=None, on_failure=None):
         """
         Creates an RQ job and saves it to Redis. The job is assigned to the
         given queue name if not None else it is assigned to scheduler queue by
@@ -138,14 +140,19 @@ class Scheduler(object):
         if kwargs is None:
             kwargs = {}
         job = self.job_class.create(
-                func, args=args, connection=self.connection,
-                kwargs=kwargs, result_ttl=result_ttl, ttl=ttl, id=id,
-                description=description, timeout=timeout, meta=meta, depends_on=depends_on)
+            func, args=args, connection=self.connection,
+            kwargs=kwargs, result_ttl=result_ttl, ttl=ttl, id=id,
+            description=description, timeout=timeout, meta=meta,
+            depends_on=depends_on,on_success=on_success,on_failure=on_failure,
+        )
         if queue_name:
             job.origin = queue_name
         else:
             job.origin = self.queue_name
 
+        if self.queue_class_name:
+            job.meta["queue_class_name"] = self.queue_class_name
+
         if commit:
             job.save()
         return job
@@ -165,6 +172,9 @@ class Scheduler(object):
         - depends_on
         - meta
         - queue_name
+        - on_success
+        - on_failure
+        - at_front
 
         Usage:
 
@@ -186,10 +196,16 @@ class Scheduler(object):
         depends_on = kwargs.pop('depends_on', None)
         meta = kwargs.pop('meta', None)
         queue_name = kwargs.pop('queue_name', None)
+        on_success = kwargs.pop('on_success', None)
+        on_failure = kwargs.pop('on_failure', None)
+        at_front = kwargs.pop('at_front', None)
 
         job = self._create_job(func, args=args, kwargs=kwargs, timeout=timeout,
                                id=job_id, result_ttl=job_result_ttl, ttl=job_ttl,
-                               description=job_description, meta=meta, queue_name=queue_name, depends_on=depends_on)
+                               description=job_description, meta=meta, queue_name=queue_name, depends_on=depends_on,
+                               on_success=on_success, on_failure=on_failure)
+        if at_front:
+            job.enqueue_at_front = True
         self.connection.zadd(self.scheduled_jobs_key,
                               {job.id: to_unix(scheduled_time)})
         return job
@@ -208,10 +224,16 @@ class Scheduler(object):
         depends_on = kwargs.pop('depends_on', None)
         meta = kwargs.pop('meta', None)
         queue_name = kwargs.pop('queue_name', None)
+        on_success = kwargs.pop('on_success', None)
+        on_failure = kwargs.pop('on_failure', None)
+        at_front = kwargs.pop('at_front', False)
 
         job = self._create_job(func, args=args, kwargs=kwargs, timeout=timeout,
                                id=job_id, result_ttl=job_result_ttl, ttl=job_ttl,
-                               description=job_description, meta=meta, queue_name=queue_name, depends_on=depends_on)
+                               description=job_description, meta=meta, queue_name=queue_name,
+                               depends_on=depends_on, on_success=on_success, on_failure=on_failure)
+        if at_front:
+            job.enqueue_at_front = True
         self.connection.zadd(self.scheduled_jobs_key,
                               {job.id: to_unix(datetime.utcnow() + time_delta)})
         return job
@@ -219,7 +241,8 @@ class Scheduler(object):
     def schedule(self, scheduled_time, func, args=None, kwargs=None,
                  interval=None, repeat=None, result_ttl=None, ttl=None,
                  timeout=None, id=None, description=None,
-                 queue_name=None, meta=None, depends_on=None):
+                 queue_name=None, meta=None, depends_on=None, on_success=None,
+                 on_failure=None, at_front=None):
         """
         Schedule a job to be periodically executed, at a certain interval.
         """
@@ -229,7 +252,8 @@ class Scheduler(object):
         job = self._create_job(func, args=args, kwargs=kwargs, commit=False,
                                result_ttl=result_ttl, ttl=ttl, id=id,
                                description=description, queue_name=queue_name,
-                               timeout=timeout, meta=meta, depends_on=depends_on)
+                               timeout=timeout, meta=meta, depends_on=depends_on,
+                               on_success=on_success, on_failure=on_failure)
 
         if interval is not None:
             job.meta['interval'] = int(interval)
@@ -237,29 +261,34 @@ class Scheduler(object):
             job.meta['repeat'] = int(repeat)
         if repeat and interval is None:
             raise ValueError("Can't repeat a job without interval argument")
+        if at_front:
+            job.enqueue_at_front = True
         job.save()
         self.connection.zadd(self.scheduled_jobs_key,
                               {job.id: to_unix(scheduled_time)})
         return job
 
     def cron(self, cron_string, func, args=None, kwargs=None, repeat=None,
-             queue_name=None, id=None, timeout=None, description=None, meta=None, use_local_timezone=False, depends_on=None):
+             queue_name=None, result_ttl=-1, ttl=None, id=None, timeout=None, description=None, meta=None, use_local_timezone=False,
+             depends_on=None, on_success=None, on_failure=None, at_front: bool = False):
         """
         Schedule a cronjob
         """
         scheduled_time = get_next_scheduled_time(cron_string, use_local_timezone=use_local_timezone)
 
-        # Set result_ttl to -1, as jobs scheduled via cron are periodic ones.
-        # Otherwise the job would expire after 500 sec.
         job = self._create_job(func, args=args, kwargs=kwargs, commit=False,
-                               result_ttl=-1, id=id, queue_name=queue_name,
-                               description=description, timeout=timeout, meta=meta, depends_on=depends_on)
+                               result_ttl=result_ttl, ttl=ttl, id=id, queue_name=queue_name,
+                               description=description, timeout=timeout, meta=meta, depends_on=depends_on,
+                               on_success=on_success, on_failure=on_failure)
 
         job.meta['cron_string'] = cron_string
         job.meta['use_local_timezone'] = use_local_timezone
 
         if repeat is not None:
             job.meta['repeat'] = int(repeat)
+        
+        if at_front:
+            job.enqueue_at_front = True
 
         job.save()
 
@@ -368,8 +397,11 @@ class Scheduler(object):
         """
         key = '{0}{1}'.format(self.queue_class.redis_queue_namespace_prefix,
                               job.origin)
-        return self.queue_class.from_queue_key(
-                key, connection=self.connection, job_class=self.job_class)
+        if job.meta.get('queue_class_name'):
+            queue_class = import_attribute(job.meta['queue_class_name'])
+        else:
+            queue_class = self.queue_class
+        return queue_class.from_queue_key(key, connection=self.connection, job_class=self.job_class)
 
     def enqueue_job(self, job):
         """
@@ -388,7 +420,7 @@ class Scheduler(object):
             job.meta['repeat'] = int(repeat) - 1
 
         queue = self.get_queue_for_job(job)
-        queue.enqueue_job(job)
+        queue.enqueue_job(job, at_front=bool(job.enqueue_at_front))
         self.connection.zrem(self.scheduled_jobs_key, job.id)
 
         if interval:
@@ -403,8 +435,9 @@ class Scheduler(object):
             if repeat is not None:
                 if job.meta['repeat'] == 0:
                     return
+            next_scheduled_time = get_next_scheduled_time(cron_string, use_local_timezone=use_local_timezone)
             self.connection.zadd(self.scheduled_jobs_key,
-                                  {job.id: to_unix(get_next_scheduled_time(cron_string, use_local_timezone=use_local_timezone))})
+                                 {job.id: to_unix(next_scheduled_time)})
 
     def enqueue_jobs(self):
         """
